import json
from pathlib import Path

CONFIG_FOLDER_PATH = Path.home() / ".adacord"


def save_auth(payload, base_path=CONFIG_FOLDER_PATH):
    Path(base_path).mkdir(exist_ok=True)
    with open(base_path / "auth.json", "w+") as f:
        f.write(json.dumps(payload))


def read_auth(base_path=CONFIG_FOLDER_PATH):
    with open(base_path / "auth.json", "r+") as f:
        return json.loads(f.read())


def get_token(base_path=CONFIG_FOLDER_PATH):
    auth = read_auth(base_path)
    return auth["token"]



from typing import Any, Dict, List, Union

import requests

from .commons import get_token
from .exceptions import AdacordApiError

HTTP_TIMEOUT = 10


class Client:
    """Low level HTTP Client interface."""

    def __init__(
        self,
        client=None,
        base_path="https://api.adacord.com/v1",
        get_token=None,
    ):
        self.client = client
        if client is None:
            self.client = requests

        self.get_token = get_token
        self.base_path = base_path

    def auth(self):
        return {"authorization": f"Bearer {self.get_token()}"}

    def post(self, path, json, auth=True):
        full_path = f"{self.base_path}{path}"

        headers = {}
        if auth:
            headers.update(self.auth())

        r = self.client.post(
            full_path, json=json, headers=headers, timeout=HTTP_TIMEOUT
        )

        if not r.ok:
            raise AdacordApiError(r.json(), status_code=r.status_code)

        return r.json()

    def get(self, path, params=None, auth=True):
        full_path = f"{self.base_path}{path}"

        headers = {}
        if auth:
            headers.update(self.auth())

        r = self.client.get(
            full_path, params=params, headers=headers, timeout=HTTP_TIMEOUT
        )

        if not r.ok:
            raise AdacordApiError(r.json(), status_code=r.status_code)

        return r.json()

    def delete(self, path, auth=True):
        full_path = f"{self.base_path}{path}"

        headers = {}
        if auth:
            headers.update(self.auth())

        r = self.client.delete(
            full_path, headers=headers, timeout=HTTP_TIMEOUT
        )

        if not r.ok:
            raise AdacordApiError(r.json(), status_code=r.status_code)

        return r.json()


class User:
    def __init__(self, instance):
        self.client = instance.client

    def create(self, email: str, password: str):
        data = {"email": email, "password": password}
        self.client.post("/users", json=data, auth=False)

    def login(self, email: str, password: str) -> Dict[str, Any]:
        data = {"email": email, "password": password}
        return self.client.post("/users/token", json=data, auth=False)


class Bucket:
    def __init__(self, instance):
        self.client = instance.client

    def create(self, description: str):
        data = {"description": description}
        return self.client.post("/buckets", json=data)

    def get(
        self, bucket: str = None
    ) -> Union[Dict[str, Any], List[Dict[str, Any]]]:
        if bucket:
            return self.client.get(f"/buckets/{bucket}")

        return self.client.get("/buckets")

    def delete(self, bucket: str) -> Dict[str, Any]:
        return self.client.delete(f"/buckets/{bucket}")

    def query(self, bucket: str, query: str):
        data = {"query": query}
        return self.client.post(f"/buckets/{bucket}/query", json=data)

    def create_webhook(
        self, bucket: str, query: str, url: str, description: str = None
    ):
        data = {"description": description, "query": query, "url": url}
        return self.client.post(f"/buckets/{bucket}/webhooks", json=data)

    def create_token(self, bucket: str, description: str = None):
        data = {"description": description}
        return self.client.post(f"/buckets/{bucket}/tokens", json=data)

    def get_tokens(self, bucket: str):
        return self.client.get(f"/buckets/{bucket}/tokens")

    def delete_token(self, bucket: str, token_uuid: str):
        return self.client.delete(f"/buckets/{bucket}/tokens/{token_uuid}")


class AdacordApi:
    def __init__(self, client=None):
        self.client = client

        self.user = User(self)
        self.bucket = Bucket(self)


api = AdacordApi(Client(get_token=get_token))



import typer

from . import commons
from .api import api
from .exceptions import cli_wrapper

app = typer.Typer()


@app.command()
@cli_wrapper
def create():
    """
    Create a new user.
    """
    typer.echo("Hey there 👋")

    email = typer.prompt("> What's your email?")
    password = typer.prompt("> What's your password?", hide_input=True)
    api.user.create(email, password)

    typer.echo(
        typer.style(
            "Awesome, check your email to confirm your email address",
            fg=typer.colors.WHITE,
            bold=True,
        )
    )


@app.command()
@cli_wrapper
def login(email: str = typer.Option(...), password: str = typer.Option(...)):
    """
    Login with the cli.
    """
    response = api.user.login(email, password)
    auth = {"email": email, "token": response["access_token"]}
    commons.save_auth(auth)



import typer
from tabulate import tabulate

from .api import api
from .exceptions import cli_wrapper

app = typer.Typer()
token_app = typer.Typer(help="Manage API Tokens.")
app.add_typer(token_app, name="token")


@app.command("create")
@cli_wrapper
def create_bucket(description: str = typer.Option("")):
    """
    Create a new bucket.
    """
    payload = api.bucket.create(description)
    typer.echo(
        typer.style(
            "Bucket created, you can start sending data 🚀",
            fg=typer.colors.WHITE,
            bold=True,
        )
    )
    first_row = ("uuid", "name", "description", "url")
    values = [payload[entry] for entry in first_row]
    first_row = ("ID", "Description", "URL")
    typer.echo(
        tabulate(
            [first_row, values], headers="firstrow", tablefmt="fancy_grid"
        )
    )


@app.command("list")
@cli_wrapper
def list_bucket():
    """
    Get a list of your buckets.
    """
    payload = api.bucket.get()
    first_row = ("uuid", "description", "url")

    if not payload:
        typer.echo(
            typer.style(
                "Ohhhh... no buckets :)", fg=typer.colors.MAGENTA, bold=True
            )
        )
        return

    rows = []
    for index, row in enumerate(payload, 1):
        rows.append([index, *[row[entry] for entry in first_row]])

    first_row = ("ID", "Description", "URL")
    first_row = ("", *first_row)
    typer.echo(
        tabulate([first_row, *rows], headers="firstrow", tablefmt="fancy_grid")
    )


@app.command("delete")
@cli_wrapper
def delete_bucket(bucket: str):
    """
    Delete a bucket.
    """
    api.bucket.delete(bucket)
    typer.echo(
        typer.style(
            f"Bucket {bucket} deleted.", fg=typer.colors.WHITE, bold=True
        )
    )


@app.command("query")
@cli_wrapper
def query_bucket(
    bucket: str = typer.Option(...), query: str = typer.Option(...)
):
    """
    Query a bucket using a SQL query.
    """
    payload = api.bucket.query(bucket, query)
    for row in payload:
        typer.echo(row)


@token_app.command("create")
@cli_wrapper
def create_token(
    bucket: str = typer.Argument(
        ..., help="The bucket uuid you want to give access to."
    ),
    description: str = typer.Option(""),
):
    """
    Create a new API Token.
    """
    payload = api.bucket.create_token(bucket, description)
    typer.echo(
        typer.style(
            "API Token created.",
            fg=typer.colors.WHITE,
            bold=True,
        )
    )
    typer.echo(
        typer.style(
            "⚠️  PLEASE SAVE THE TOKEN, THIS IS THE ONLY TIME WHERE WE DISPLAY IT ⚠️",
            fg=typer.colors.WHITE,
            bold=True,
        )
    )
    first_row = ("uuid", "token", "description", "created_on")
    values = [payload[entry] for entry in first_row]
    first_row = ("ID", "Token", "Description", "Created on")
    typer.echo(
        tabulate(
            [first_row, values], headers="firstrow", tablefmt="fancy_grid"
        )
    )


@token_app.command("list")
@cli_wrapper
def list_tokens(
    bucket: str = typer.Argument(
        ..., help="The bucket uuid you want to give access to."
    )
):
    """
    Get your tokens.
    """
    tokens = api.bucket.get_tokens(bucket)
    first_row = ("uuid", "token", "description", "created_on")
    if not tokens:
        typer.echo(
            typer.style(
                f"No tokens for Bucket({bucket}).",
                fg=typer.colors.WHITE,
                bold=True,
            )
        )
        return

    rows = []
    for index, row in enumerate(tokens, 1):
        rows.append([index, *[row[entry] for entry in first_row]])

    first_row = ("ID", "Token", "Description", "Created on")
    first_row = ("", *first_row)
    typer.echo(
        tabulate([first_row, *rows], headers="firstrow", tablefmt="fancy_grid")
    )


@token_app.command("delete")
@cli_wrapper
def delete_token(bucket: str, token: str):
    """
    Delete an API Token.
    """
    api.bucket.delete_token(bucket, token)
    typer.echo(
        typer.style(
            f"API Token {token} deleted.", fg=typer.colors.WHITE, bold=True
        )
    )
